# passerelle.contrib.teamnet_axel
# Copyright (C) 2015  Entr'ouvert
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import logging
import xml.etree.ElementTree as ET

from django.core.urlresolvers import reverse
from django.db import models
from django.utils.translation import ugettext_lazy as _

from passerelle.base.models import BaseResource

from . import soap

logger = logging.getLogger('passerelle.contrib.teamnet_axel')


class AxelException(Exception):
    pass


class Management(BaseResource):
    wsdl_url = models.CharField(max_length=128, blank=False,
            verbose_name=_('WSDL URL'),
            help_text=_('Teamnet Axel WSDL URL'))
    verify_cert = models.BooleanField(default=True,
            verbose_name=_('Check HTTPS Certificate validity'))
    username = models.CharField(max_length=128, blank=True,
            verbose_name=_('Username'))
    password = models.CharField(max_length=128, blank=True,
            verbose_name=_('Password'))
    keystore = models.FileField(upload_to='teamnet_axel', null=True, blank=True,
            verbose_name=_('Keystore'),
            help_text=_('Certificate and private key in PEM format'))

    category = _('Business Process Connectors')

    class Meta:
        verbose_name = _('Teamnet Axel')

    @classmethod
    def get_icon_class(cls):
        return 'ressources'

    @classmethod
    def get_verbose_name(cls):
        return cls._meta.verbose_name

    def get_absolute_url(self):
        return reverse('teamnet-axel-view', kwargs={'slug': self.slug})

    @classmethod
    def get_add_url(cls):
        return reverse('teamnet-axel-add')

    #
    # Axel SOAP call: getData
    #
    def get_data(self, operation, args):
        # args is a XML node (ElementTree.Element)
        portail = ET.Element('PORTAIL')
        portail.append(args)
        streamId = operation
        xmlParams = ET.tostring(portail, encoding='UTF-8')
        user = ''
        logger.debug('getData(streamId=%s, xmlParams=%s, user=%s)',
                streamId, xmlParams, user)
        result = soap.get_client(self).service.getData(streamId, xmlParams, user)
        logger.debug('getData(%s) result: %s', streamId, result)
        xml_result = ET.fromstring(result)
        if xml_result.find('RESULTAT/STATUS').text != 'OK':
            msg = xml_result.find('RESULTAT/COMMENTAIRES').text
            raise AxelException(msg)
        return xml_result.find('DATA')

    # Axel authentication
    def auth(self, login, pwd):
        '''return False or an AXEL user dict:
          {
              "login": "23060A",
              "estidentifie": true,
              "estbloque": false,
              "estchangement_mdp_requis": false,
              "nbechec": "0",
              "idpersonne": "47747",
              "idfamille": "23060",
          }
        '''
        xml_utilisateur = ET.Element('UTILISATEUR')
        xml_login = ET.Element('LOGIN')
        xml_login.text = login
        xml_utilisateur.append(xml_login)
        xml_pwd = ET.Element('PWD')
        xml_pwd.text = pwd
        xml_utilisateur.append(xml_pwd)
        data = self.get_data('ConnexionCompteFamille', xml_utilisateur)
        data = data.find('PORTAIL/UTILISATEUR')
        data = soap.xml_to_dict(data)
        for key, value in data.items():
            if key.startswith('est'):
                data[key] = value == 'true'
        if data.get('estbloque'):
            return False
        if not data.get('estidentifie'):
            return False
        return data

    def get_famille(self, idfamille, annee=None):
        xml_famille = ET.Element('FAMILLE')
        xml_idfamille = ET.Element('IDFAMILLE')
        xml_idfamille.text = idfamille
        xml_famille.append(xml_idfamille)
        if annee:
            xml_annee = ET.Element('ANNEE')
            xml_annee.text = '%s' % annee
            xml_famille.append(xml_annee)
        data = self.get_data('DonneesFamille', xml_famille)
        xml_individus = data.findall('PORTAIL/INDIVIDUS')
        if not xml_individus:
            raise AxelException('PORTAIL/INDIVIDUS is empty')
        individus = [dict((k.lower(), v) for k, v in i.attrib.items()) for i in xml_individus]
        for individu in individus:
            individu['id'] = individu['idindividu']
            individu['text'] = '%(prenom)s %(nom)s' % individu
        responsables = [i for i in individus if i['indtype'] in ('1', '2')]
        enfants = [i for i in individus if i['indtype'] == '3']
        return {
            'responsables': responsables,
            'enfants': enfants,
        }


class Link(models.Model):
    resource = models.ForeignKey(Management)
    nameid = models.CharField(blank=False, max_length=256)
    login = models.CharField(blank=False, max_length=128)
    pwd = models.CharField(blank=False, max_length=128)
